import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.lang.*;
import java.text.DecimalFormat;

//--------------------------------------------------------------------//
/**
 * ANVǗNX
 */
public class PendulumActionManager
  implements Runnable,MouseListener,MouseMotionListener{

/** px */
  public static final int N=120;

/** px̌x */
  public static final double THETA=Math.toRadians(170.0);

/** l */
  protected static final DecimalFormat
    DFORM=new DecimalFormat("#######0.000");

/** \pl */
  protected PendulumViewPanel panel;

/** Aj[V\Ԋu */
  protected int waitTime;

/** px */
  protected double theta0;

/** Uq̎ */
  protected double period;

/** ԑ */
  protected double dt;

/** x */
  protected double cv;

/** Uq^ */
  protected Motion motion;

/** }EXʒu\ <code>Point</code> */
  protected Point mouse;

/** \XVXbh */
  protected Thread thread;

/**
 * ANVǗIuWFNg\z
 * @param panel \pl
 */
  public PendulumActionManager(PendulumViewPanel panel){
    this.panel=panel;
    panel.addMouseListener(this);
    panel.addMouseMotionListener(this);
    mouse=new Point();
    thread=null;
    motion=new Motion();
    theta0=0.0;
    period=0.0;
    dt=0.0;
    cv=0.0;
    waitTime=0;
  }

//--------------------------------------------------------------------//
/**
 * Xbh̎sJn
 */
  public void start(){
    if(thread==null){
      thread=new Thread(this);
      thread.start();
    }
  }

//--------------------------------------------------------------------//
/**
 * Xbhs
 */
  public void run(){
    synchronized(this){
      while(thread!=null){
      	PendulumMotion.nextRKG(dt,cv,motion);
      	panel.repaint();
      	try{
      	  Thread.sleep(waitTime);
      	}
      	catch(InterruptedException ex){
      	}
      }
    }
  }

//--------------------------------------------------------------------//
/**
 * Xbh̎s~
 */
  public void stop(){
    thread=null;
  }

//--------------------------------------------------------------------//
/**
 * }EX{^̏s
 * @param e }EXANVƂCxg
 */
  public void mousePressed(MouseEvent e){
    stop();
    mouse.setLocation(e.getX(),e.getY());
  }

//--------------------------------------------------------------------//
/**
 * }EX{^𗣂̏s
 * @param e }EXANVƂCxg
 */
  public void mouseReleased(MouseEvent e){
  }

//--------------------------------------------------------------------//
/**
 * }EXNbN̏s
 * @param e }EXANVƂCxg
 */
  public void mouseClicked(MouseEvent e){
  }

//--------------------------------------------------------------------//
/**
 * }EXplɓ̏s
 * @param e }EXANVƂCxg
 */
  public void mouseEntered(MouseEvent e){
  }

//--------------------------------------------------------------------//
/**
 * }EXplȍs
 * @param e }EXANVƂCxg
 */
  public void mouseExited(MouseEvent e){
  }

//--------------------------------------------------------------------//
/**
 * }EXړ̏s
 * @param e }EXANVƂCxg
 */
  public void mouseMoved(MouseEvent e){
  }

//--------------------------------------------------------------------//
/**
 * }EXhbȌs
 * @param e }EXANVƂCxg
 */
  public void mouseDragged(MouseEvent e){
    stop();
    int x=e.getX();
    int y=e.getY();
    theta0+=Math.toRadians((double)(mouse.x-x));
    theta0=Math.min(Math.max(theta0,-THETA),THETA);
    mouse.setLocation(x,y);
    motion.setTheta(theta0);
    panel.setPaint();
    panel.repaint();
  }

//--------------------------------------------------------------------//
/**
 * p[^ݒ肷
 * @param length Uq
 * @param g d͉x
 */
  public void setParameters(double length,double g){
    panel.setPaint();
    period=PendulumMotion.period(length,g,theta0);
    dt=period/(double)(N-1);
    cv=g*dt/length;
    waitTime=(int)(1000.0*dt);
    motion.setTime(0.0);
    motion.setTheta(theta0);
    motion.setOmega(0.0);
    start();
  }

//--------------------------------------------------------------------//
/**
 * Uq̏pxԂ
 * @return Uq̏px
 */
  public double getInitialAngle(){
    return theta0;
  }

//--------------------------------------------------------------------//
/**
 * Uq̊pxԂ
 * @return Uq̊px
 */
  public double getAngle(){
    return motion.theta;
  }

//--------------------------------------------------------------------//
/**
 * Uq^Ԃ
 * @return Uq^
 */
  public String info(){
    StringBuffer info=new StringBuffer("period: ");
    info.append(DFORM.format(period));
    info.append("  time:");
    info.append(DFORM.format(motion.time));
    info.append("  initial angle:");
    info.append(DFORM.format(Math.toDegrees(theta0)));
    return info.toString();
  }
}
