import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Panel;
import java.awt.Point;
import java.awt.image.VolatileImage;
import java.lang.*;

//--------------------------------------------------------------------//
/**
 * \pl
 */
public class PendulumViewPanel extends Panel{

/** plTCY */
  protected Dimension panelSize;

/** UqS\ʒu */
  protected Point center;

/** Uq\ */
  protected double length;

/** Uq\ʒu */
  protected Point infoPoint;

/** _uobt@OpC[W */
  protected VolatileImage bgImage;

/** _uobt@OpC[W̃OtBbNXReLXg */
  protected Graphics2D bg;

/** wi\F */
  protected Color bgColor;

/** Oi\F */
  protected Color fgColor;

/** tHg */
  protected Font font;

/** ANVǗIuWFNg */
  protected PendulumActionManager manager;

/**
 * \pl\z
 * @param bgColor wi\F
 * @param fgColor Oi\F
 * @param font tHg
 */
  public PendulumViewPanel(Color bgColor,Color fgColor,Font font){
    this.bgColor=bgColor;
    this.fgColor=fgColor;
    this.font=font;
    setBackground(bgColor);
    manager=new PendulumActionManager(this);
  }

//--------------------------------------------------------------------//
/**
 * p[^ݒ肷
 * @param length Uq
 * @param g d͉x
 */
  public void setParameters(double length,double g){
    setPaint();
    manager.setParameters(length,g);
  }

//--------------------------------------------------------------------//
/**
 * ĕ`悷
 * @param g `悷OtBbNXReLXg
 */
  public void update(Graphics g){
    drawPendulum(bg);
    g.drawImage(bgImage,0,0,this);
  }

//--------------------------------------------------------------------//
/**
 * `悷
 * @param g `悷OtBbNXReLXg
 */
  public void paint(Graphics g){
    setPaint();
    drawPendulum(bg);
    g.drawImage(bgImage,0,0,this);
  }

//--------------------------------------------------------------------//
/**
 * ʂݒ肷
 */
  protected void setPaint(){
    panelSize=getSize();
    center=new Point(panelSize.width/2,panelSize.height/6);
    double theta0=manager.getInitialAngle();
    double maxWidth;
    if(theta0==0.0){
      maxWidth=(double)panelSize.height;
    }
    else{
      maxWidth=0.8*(double)center.x/Math.abs(Math.sin(theta0));
    }
    length=Math.min(2.0*(double)panelSize.height/3.0,maxWidth);
    if(Math.abs(theta0)>0.5*Math.PI){
      double cc0=Math.cos(theta0);
      double dlength=length*cc0/(cc0-1.0);
      center.y+=dlength;
      length-=dlength;
    }
    bgImage=createVolatileImage(panelSize.width,panelSize.height);
    bg=bgImage.createGraphics();
    bg.setFont(font);
    FontMetrics fm=bg.getFontMetrics();
    infoPoint=new Point(20,fm.getHeight()+2);
  }

//--------------------------------------------------------------------//
/**
 * Uq`悷
 * @param g `悷OtBbNXReLXg
 */
  protected void drawPendulum(Graphics2D g){
    int r=5;
    int d=2*r;
    g.setColor(bgColor);
    g.fillRect(0,0,panelSize.width,panelSize.height);
    g.setColor(fgColor);
    g.drawString(manager.info(),infoPoint.x,infoPoint.y);
    double theta=manager.getAngle();
    int px=center.x-(int)(length*Math.sin(theta));
    int py=center.y+(int)(length*Math.cos(theta));
    g.drawLine(center.x,center.y,px,py);
    g.fillOval(px-r,py-r,d,d);
  }
}
